<?php

namespace Drupal\commerce_icici_pay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_icici_pay\IciciUtility;

/**
 * Provides the ICICI offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "icici_redirect",
 *   label = @Translation("ICICI Redirect"),
 *   display_label = @Translation("ICICI"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_icici_pay\PluginForm\IciciRedirect\IciciPaymentForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 * )
 */
class IciciRedirect extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'merchant_id' => '',
        'secret_key' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant Id'),
      '#description' => $this->t('This is the Merchant Id.'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#description' => $this->t('This is the Secret Key.'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['merchant_id'] = $values['merchant_id'];
    $this->configuration['secret_key'] = $values['secret_key'];
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $order_status = 'Failure';
    $order_price = '';
    $resp_order_id = '';
    $payment_mode = $payment_id = $transaction_id = '';
    $request_data = $request->request->all();

    if (!empty($request_data)) {
      $response_code = $request_data['responseCode'];
      $order_price = $request_data['amount'];
      $payment_mode = $request_data['paymentMode'];
      $payment_id = $request_data['paymentID'];
      $transaction_id = $request_data['txnID'];
      $resp_order_id = preg_replace('/[^0-9]/', '', $request_data['invoiceNo']);
      if ($response_code == '000' || $response_code == '0000') {
        $order_status = 'Success';
      }
    }

    // Get secret key from configuration
    $working_key = $this->configuration['secret_key'];

    // Validate secure hash
    $utility = new IciciUtility();
    $validated = $utility->validateSecureHash($request_data, $working_key);
    if ($validated == false) {
      drupal_set_message($this->t('Security Error. Illegal access detected.'), 'error');
      throw new \Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException();
    }

    switch ($order_status) {

      case 'Success':
        drupal_set_message($this->t('Your payment was successful with Voucher Id : @voucherid and Transaction id : @transaction_id', ['@voucherid' => $request_data['invoiceNo'], '@transaction_id' => $transaction_id]));
        // Code to handle payment status success
        break;

      case 'Failure':
        // Code to handle payment status failed
        drupal_set_message($this->t('The transaction has been failed.'), 'error');
        break;

      default:
        drupal_set_message($this->t('Security Error. Illegal access detected.'), 'error');
        break;
    }

  }

}